#!/usr/bin/env bash

# Script to check whether build.sh would result in a different  set of files.
# This is currently only used in the CI.
# Placed in the docker directory because it can be refactored to provision
# the docker image.

# https://betterdev.blog/minimal-safe-bash-script-template/
# Cannot use -u due to openjdk:
# https://github.com/conda-forge/openjdk-feedstock/issues/87
set -Eeo pipefail
set -x

# Parse command line options, --delete will delete auto-generated files.
while [[ $# -gt 0 ]]; do
  case $1 in
    --delete)
      DELETE=YES
      shift # past argument
      ;;
    *)
      POSITIONAL_ARGS+=("$1") # save positional arg
      shift # past argument
      ;;
  esac
done


# Build the files locally. As a safety precaution, only delete files when
# explicitly asked to do so.
if [ "${DELETE}" = "YES" ] ; then
    echo "Going to delete auto-generated files!"
    bash build.sh --delete
else
    bash build.sh
fi

# Exit with an error code if there are still differences left.
git diff --exit-code
