
from edps.generator.task_builder import data_source, task
from edps.generator.task import ClassificationRule, ProductClassificationRule


def is_raw(f):
    return f["dpr.catg"] == "NODEFAULT" and f["dpr.tech"] == "" and f["dpr.type"] == ""

raw_class = ClassificationRule("RAW", is_raw)

raw = (
    data_source()
    .with_classification_rule(raw_class)
    .build()
)


def is_raw_dark(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "DARK"

raw_dark_class = ClassificationRule("I_DARK_RAW", is_raw_dark)

raw_dark = (
    data_source()
    .with_classification_rule(raw_dark_class)
    .build()
)







def is_cat_photomref(f):
    return f["pro.catg"] == "CAT_PHOTOMREF"

cat_photomref_class = ClassificationRule("REF_I_PHOTOM_CAT", is_cat_photomref)

cat_photomref = (
    data_source()
    .with_classification_rule(cat_photomref_class)
    .build()
)





def is_raw_opt_flat(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "FLAT,LAMP"

raw_opt_flat_class = ClassificationRule("I_FLAT_RAW", is_raw_opt_flat)

raw_opt_flat = (
    data_source()
    .with_classification_rule(raw_opt_flat_class)
    .build()
)


def is_raw_opt_nonlin(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "NONLIN,LAMP"

raw_opt_nonlin_class = ClassificationRule("I_NONLIN_RAW", is_raw_opt_nonlin)

raw_opt_nonlin = (
    data_source()
    .with_classification_rule(raw_opt_nonlin_class)
    .build()
)



def is_cat_distortionref_astromref(f):
    return f["pro.catg"] == "CAT_DISTORTIONREF_ASTROMREF"

cat_distortionref_astromref_class = ClassificationRule("REF_I_ASTROM_CAT", is_cat_distortionref_astromref)

cat_distortionref_astromref = (
    data_source()
    .with_classification_rule(cat_distortionref_astromref_class)
    .build()
)


def is_cat_distortionref_wamref(f):
    return f["pro.catg"] == "CAT_DISTORTIONREF_WAMREF"

cat_distortionref_wamref_class = ClassificationRule("REF_I_WAM_CAT", is_cat_distortionref_wamref)

cat_distortionref_wamref = (
    data_source()
    .with_classification_rule(cat_distortionref_wamref_class)
    .build()
)






def is_raw_opt_sky_astromfield(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "STD,ASTROMETRY"

raw_opt_sky_astromfield_class = ClassificationRule("I_ASTROMFIELD_RAW", is_raw_opt_sky_astromfield)

raw_opt_sky_astromfield = (
    data_source()
    .with_classification_rule(raw_opt_sky_astromfield_class)
    .build()
)


def is_raw_opt_sky_bkgfield(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,AI" and f["dpr.type"] == "OBJECT,BCKGRND"

raw_opt_sky_bkgfield_class = ClassificationRule("I_BKGFIELD_RAW", is_raw_opt_sky_bkgfield)

raw_opt_sky_bkgfield = (
    data_source()
    .with_classification_rule(raw_opt_sky_bkgfield_class)
    .build()
)


def is_raw_opt_wamcam_cam(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,AI" and f["dpr.type"] == "PINHOLE,CAM"

raw_opt_wamcam_cam_class = ClassificationRule("I_CAM_RAW", is_raw_opt_wamcam_cam)

raw_opt_wamcam_cam = (
    data_source()
    .with_classification_rule(raw_opt_wamcam_cam_class)
    .build()
)


def is_raw_opt_sky_illum(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "STD,ILLUM"

raw_opt_sky_illum_class = ClassificationRule("I_ILLUM_RAW", is_raw_opt_sky_illum)

raw_opt_sky_illum = (
    data_source()
    .with_classification_rule(raw_opt_sky_illum_class)
    .build()
)


def is_raw_opt_sky_stdfield(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "STD,PHOTOM"

raw_opt_sky_stdfield_class = ClassificationRule("I_STDFIELD_RAW", is_raw_opt_sky_stdfield)

raw_opt_sky_stdfield = (
    data_source()
    .with_classification_rule(raw_opt_sky_stdfield_class)
    .build()
)


def is_raw_opt_wamcam_wam(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "IMAGE,AI" and f["dpr.type"] == "PINHOLE,WAM"

raw_opt_wamcam_wam_class = ClassificationRule("I_WAM_RAW", is_raw_opt_wamcam_wam)

raw_opt_wamcam_wam = (
    data_source()
    .with_classification_rule(raw_opt_wamcam_wam_class)
    .build()
)



def is_raw_opt_sky_skysiai_science(f):
    return f["dpr.catg"] == "SCIENCE" and f["dpr.tech"] == "IMAGE,SI" and f["dpr.type"] == "OBJECT,SKY"

raw_opt_sky_skysiai_science_class = ClassificationRule("I_SCI_RAW", is_raw_opt_sky_skysiai_science)

raw_opt_sky_skysiai_science = (
    data_source()
    .with_classification_rule(raw_opt_sky_skysiai_science_class)
    .build()
)


def is_raw_opt_sky_skysiai_scienceai(f):
    return f["dpr.catg"] == "SCIENCE" and f["dpr.tech"] == "IMAGE,AI" and f["dpr.type"] == "OBJECT,SKY"

raw_opt_sky_skysiai_scienceai_class = ClassificationRule("I_SCI_AI_RAW", is_raw_opt_sky_skysiai_scienceai)

raw_opt_sky_skysiai_scienceai = (
    data_source()
    .with_classification_rule(raw_opt_sky_skysiai_scienceai_class)
    .build()
)











def is_raw_opt_sky_skyhci_coro_corobackground(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "CORONOGRAPHY" and f["dpr.type"] == "BACKGROUND"

raw_opt_sky_skyhci_coro_corobackground_class = ClassificationRule("I_CORO_BGD_RAW", is_raw_opt_sky_skyhci_coro_corobackground)

raw_opt_sky_skyhci_coro_corobackground = (
    data_source()
    .with_classification_rule(raw_opt_sky_skyhci_coro_corobackground_class)
    .build()
)


def is_raw_opt_sky_skyhci_coro_corooffset(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "CORONOGRAPHY" and f["dpr.type"] == "STAR"

raw_opt_sky_skyhci_coro_corooffset_class = ClassificationRule("I_CORO_OFFSET_RAW", is_raw_opt_sky_skyhci_coro_corooffset)

raw_opt_sky_skyhci_coro_corooffset = (
    data_source()
    .with_classification_rule(raw_opt_sky_skyhci_coro_corooffset_class)
    .build()
)


def is_raw_opt_sky_skyhci_coro_sciencecoropupil(f):
    return f["dpr.catg"] == "SCIENCE" and f["dpr.tech"] == "CORONOGRAPHY" and f["dpr.type"] == "PUPIL"

raw_opt_sky_skyhci_coro_sciencecoropupil_class = ClassificationRule("I_SCI_CORO_PUPIL_RAW", is_raw_opt_sky_skyhci_coro_sciencecoropupil)

raw_opt_sky_skyhci_coro_sciencecoropupil = (
    data_source()
    .with_classification_rule(raw_opt_sky_skyhci_coro_sciencecoropupil_class)
    .build()
)


def is_raw_opt_sky_skyhci_coro_sciencecorowaffle(f):
    return f["dpr.catg"] == "SCIENCE" and f["dpr.tech"] == "CORONOGRAPHY,WAFFLE" and f["dpr.type"] == "OBJECT"

raw_opt_sky_skyhci_coro_sciencecorowaffle_class = ClassificationRule("I_SCI_CORO_WAFFLE_RAW", is_raw_opt_sky_skyhci_coro_sciencecorowaffle)

raw_opt_sky_skyhci_coro_sciencecorowaffle = (
    data_source()
    .with_classification_rule(raw_opt_sky_skyhci_coro_sciencecorowaffle_class)
    .build()
)






def is_raw_opt_sky_skyhci_skyhcionline_dpsicsoptsci_dpsicsopt(f):
    return f["dpr.catg"] == "CALIB" and f["dpr.tech"] == "CORONOGRAPHY" and f["dpr.type"] == "HCIOPT"

raw_opt_sky_skyhci_skyhcionline_dpsicsoptsci_dpsicsopt_class = ClassificationRule("I_DPSICS_OPT_RAW", is_raw_opt_sky_skyhci_skyhcionline_dpsicsoptsci_dpsicsopt)

raw_opt_sky_skyhci_skyhcionline_dpsicsoptsci_dpsicsopt = (
    data_source()
    .with_classification_rule(raw_opt_sky_skyhci_skyhcionline_dpsicsoptsci_dpsicsopt_class)
    .build()
)



img_masterdark = (
    task('micado_det_dark')
    .with_main_input(raw_dark)

    .build()
)


img_masterflat = (
    task('micado_img_flat')
    .with_main_input(raw_opt_flat)
    .with_associated_input(img_masterdark)
    .build()
)


img_nonlinearity = (
    task('micado_img_nonlinearity')
    .with_main_input(raw_opt_nonlin)

    .build()
)


img_persistence = (
    task('micado_det_persistence')
    .with_main_input(raw)
    .with_associated_input(raw)
    .build()
)


img_fnl_catalogable_dt_dtsiai_dt = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_skysiai_science)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_fnl_catalogable_dt_dtbkgfield = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_bkgfield)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_background = (
    task('micado_img_background')
    .with_main_input(img_fnl_catalogable_dt_dtbkgfield)

    .build()
)


img_fnl_catalogable_dt_dtillum = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_illum)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


hdr_statictable_illumcorr = (
    task('micado_img_illum')
    .with_main_input(img_fnl_catalogable_dt_dtillum)
    .with_associated_input(cat_photomref)
    .build()
)


img_fnl_catalogable_dtfnl_dtfnlsiai_dtfnl = (
    task('micado_img_detrend_final')
    .with_main_input(img_fnl_catalogable_dt_dtsiai_dt)
    .with_associated_input(img_background)
    .with_associated_input(hdr_statictable_illumcorr)
    .build()
)


img_fnl_catalogable_dt_dtsiai_dtai = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_skysiai_scienceai)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_fnl_catalogable_dtfnl_dtfnlsiai_dtfnlai = (
    task('micado_img_detrend_final')
    .with_main_input(img_fnl_catalogable_dt_dtsiai_dtai)
    .with_associated_input(img_background)
    .with_associated_input(hdr_statictable_illumcorr)
    .build()
)


img_fnl_catalogable_dt_dtwam = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_wamcam_wam)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_fnl_catalogable_dtfnl_dtfnlwam = (
    task('micado_img_detrend_final')
    .with_main_input(img_fnl_catalogable_dt_dtwam)
    .with_associated_input(img_background)
    .with_associated_input(hdr_statictable_illumcorr)
    .build()
)


hdr_distortion_distortionwam = (
    task('micado_img_distortion')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlwam)
    .with_associated_input(cat_distortionref_wamref)
    .build()
)


img_fnl_catalogable_dt_dtastromfield = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_astromfield)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_fnl_catalogable_dtfnl_dtfnlastromfield = (
    task('micado_img_detrend_final')
    .with_main_input(img_fnl_catalogable_dt_dtastromfield)
    .with_associated_input(img_background)
    .with_associated_input(hdr_statictable_illumcorr)
    .build()
)


hdr_distortion_distortionelt = (
    task('micado_img_distortion')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlastromfield)
    .with_associated_input(cat_distortionref_astromref)
    .build()
)


hdr_astrom = (
    task('micado_img_astrom')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlsiai_dtfnl)
    .with_associated_input(cat_distortionref_astromref)
    .with_associated_input(hdr_distortion_distortionwam)
    .with_associated_input(hdr_distortion_distortionelt)
    .build()
)


hdr_photom_photomscience = (
    task('micado_img_photom')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlsiai_dtfnl)
    .with_associated_input(hdr_astrom)
    .with_associated_input(cat_photomref)
    .build()
)


img_fnl_catalogable_dt_dtstdfield = (
    task('micado_img_detrend')
    .with_main_input(raw_opt_sky_stdfield)
    .with_associated_input(img_masterdark)
    .with_associated_input(img_masterflat)
    .with_associated_input(img_nonlinearity)
    .with_associated_input(img_persistence)
    .build()
)


img_fnl_catalogable_dtfnl_dtfnlstdfield = (
    task('micado_img_detrend_final')
    .with_main_input(img_fnl_catalogable_dt_dtstdfield)
    .with_associated_input(img_background)
    .with_associated_input(hdr_statictable_illumcorr)
    .build()
)


hdr_photom_photomstdfield = (
    task('micado_img_photom')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlstdfield)
    .with_associated_input(hdr_astrom)
    .with_associated_input(cat_photomref)
    .build()
)


img_fnl_catalogable_calibrated = (
    task('micado_img_calibrate')
    .with_main_input(img_fnl_catalogable_dtfnl_dtfnlsiai_dtfnl)
    .with_associated_input(hdr_photom_photomscience)
    .with_associated_input(hdr_astrom)
    .build()
)


img_stack = (
    task('micado_img_stack')
    .with_main_input(img_fnl_catalogable_calibrated)

    .build()
)


