"""Methods for the MicadoWISE Environment."""

from os import environ, path
from pathlib import Path

# Copy of the logic in commonwrapper. This is necessary because Environment.py
# should not create a database connection, because the dpu runner will import
# the Environment to set the database credentials.
MOCKCOMMON = environ.get("mockcommon", "")

try:
    import common
except ImportError:
    MOCKCOMMON = "True"

if not MOCKCOMMON:
    from common.config.Environment import Env, ConfigFile
else:
    from micado.main.commonwrapper import Env, ConfigFile


def init_micado_env():
    """Initialize MICADO specific Env."""
    configfiles = [
        pathname
        for pathname in (
            path.join(path.dirname(__file__), "Environment.cfg"),
            path.join(str(Path.home()), ".awe", "Environment.cfg"),
            path.join(str(Path.home()), ".awe", "micado", "Environment.cfg"),
        )
        if path.exists(pathname)
    ]
    for configfile in configfiles:
        cfg = ConfigFile(configfile)
        Env.update(cfg.as_dict())

    for key in Env.keys():
        if key in environ:
            Env[key] = environ[key]


init_micado_env()
