# -*- coding: utf-8 -*-
"""Telescope.

This module is part of the hardware description.
All hardware clasess have a one-to-one correspondence with a class in
the irdb: https://github.com/AstarVienna/irdb

The base Hardware class has two subclasses:
- HardwareThing, which is Atmosphere/Telescope/Detector/... etc.
- Effect, which is Shotnoise/Nonlinearity/... etc.

These classes also provide default instances. The values of those
instances are derived from the IRDB.

In the future it might be better to generate these classes directly
from the IRDB.

This file is automatically generated. Do not modify!
"""

from micado.main.HardwareThing import HardwareThing, persistent
from micado.hardware.Reflection import Reflection, default_reflection


class Telescope(HardwareThing):
    """The Telescope."""

    name = persistent("Name of telescope [TEL NAME][None]", str, "ELT")

    telescope = persistent("Name of telescope [TEL TELESCOP][None]", str, "ELT")

    ter_curve__filename = persistent(
        "File [TEL TER FILENAME][None]", str, "TER_ELT_5_mirror.dat"
    )

    reflection = persistent(
        "single combined reflection curve (ESO-333023) [][None]",
        Reflection,
        default_reflection,
    )


default_telescope = Telescope(
    name="ELT",
    telescope="ELT",
    ter_curve__filename="TER_ELT_5_mirror.dat",
    reflection=default_reflection,
)
