# -*- coding: utf-8 -*-
"""Terminology as used in the ESO archive."""

from enum import Flag, auto


# TODO: Generate these automatically from VODML
class DprType(Flag):
    """Type of the observation. Multiple choices are allowed.

    Based on
    - http://archive.eso.org/eso/eso_archive_help.html#DPR_type
    - Table 10 in
    http://archive.eso.org/cms/tools-documentation/dicb/ESO-044156_6DataInterfaceControlDocument.pdf
    -
    https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500
    """

    # Principal Values:
    # Any observation of an unspecified object
    OBJECT = auto()
    # Dark exposure
    DARK = auto()
    # Any flat field exposure
    FLAT = auto()
    SLIT = auto()
    COLDMASK = auto()
    WARMMASK = auto()
    LAMPPUPIL = auto()
    WAFFLE = auto()

    # Qualifiers:
    # Any observation of an empty field on the sky
    SKY = auto()
    # Any lamp exposure
    LAMP = auto()
    # Any exposure using the dome
    DOME = auto()
    # Any exposure using an illuminated screen
    SCREEN = auto()
    # Flux standard (spectroscopy and photometry)
    FLUX = auto()
