# -*- coding: utf-8 -*-
"""Tests for getting Darks in and out of the database."""

from common.database.Context import context
from common.database.Database import database
from micado.dataitems.Dark import Dark
from micado.dataitems.MasterDark import MasterDark
from micado.recipes.McdDetDark import McdDetDark
from micado.test.functional.dark_test import generate_dark
from micado.test.functional.masterdark_test import (
    create_masterdark,
)


def test_darkdb():
    """Tests loading pixels of a Dark from the database."""
    query_dark = Dark.select_all()
    len_query_1 = len(query_dark)

    mydark1 = generate_dark()
    mydark1.save_fits()
    mydark1.commit()

    len_query_2 = len(query_dark)
    assert len_query_2 == len_query_1 + 1

    # Clear object cache so the Dark can be fetched again.
    database.clear_objcache()

    mydark2 = Dark(object_id=mydark1.object_id)

    # At the moment two objects are not equivalent if the pixels aren't loaded.
    # Now the pixels are loaded, but this will not scale, so needs to be fixed.
    # TODO: Should they be equivalent?
    # assert mydark2 != mydark1

    # Load the pixels.
    mydark2.load_data()

    # Now they should be the same. Note that this traverses the full dependency
    # graph.
    assert mydark2 == mydark1
    context.delete(mydark2, commit=True)


def test_masterdarkdb():
    """Tests loading pixels of a Dark from the database."""

    mymasterdark1, _, mydarks1 = create_masterdark()

    for dpd in mydarks1:
        dpd.commit()

    # Cache object ids.
    object_ids_dpds = [dpd.object_id for dpd in mydarks1]

    # Clear object cache so the Dark can be fetched again.
    database.clear_objcache()

    # Now act like we just happened to have a set of object ids of
    # darks that we want to make a masterdark from.

    mydarks2 = [Dark(object_id=oid) for oid in object_ids_dpds]

    # A recipe. The dark recipe has no parameters.
    myrecipe = McdDetDark()

    # Create MasterDark.
    mymasterdark2 = MasterDark(
        recipe=myrecipe,
        darks=mydarks2,
    )

    mymasterdark2.make_proto()
    # Need to save in order to get the filename right.
    mymasterdark2.save_fits()

    assert mymasterdark1 == mymasterdark2
