"""Test fitsfloat functionality."""
import pytest
from micado.dataitems.Dark import Dark
from micado.main.ObservingBlock import ObservingBlock
from micado.templates.MicadoImgCalDark import MicadoImgCalDark
from micado.test.functional.dark_test import remove_file_and_links
from micado.util.fitsfloat import fitsfloat


def test_fitsfloat_failure():
    """Test truncation of floats by astropy."""
    mydark1 = Dark(
        det_dit=1234567891234.1235,
        template=MicadoImgCalDark(
            det__dit=1234567891234.1235,
        ),
        observing_block=ObservingBlock(templates=[MicadoImgCalDark()]),
    )
    mydark1.save_fits()
    mydark2 = Dark(filename=mydark1.filename)
    print(mydark1.get_differences(mydark2))
    assert mydark1.det_dit != mydark2.det_dit
    remove_file_and_links(mydark1.filename)


@pytest.mark.xfail(reason="TODO: Apparently reading DET.DIT from FITS is broken.")
def test_fitsfloat_success():
    """Test fitsfloat fix."""
    mydark1 = Dark(
        det_dit=fitsfloat(1234567891234.1235),
        template=MicadoImgCalDark(
            det__dit=fitsfloat(1234567891234.1235),
        ),
        observing_block=ObservingBlock(templates=[MicadoImgCalDark()]),
    )
    mydark1.save_fits()
    mydark2 = Dark(filename=mydark1.filename)
    print(mydark1.get_differences(mydark2))
    assert mydark1.det_dit == mydark2.det_dit
    assert mydark1 == mydark2
    remove_file_and_links(mydark1.filename)
