from datetime import datetime

from common.database.Context import context
from micado.dataitems.Dark import Dark
from micado.dataitems.DarkExtension import DarkExtension
from micado.main.ObservingBlock import ObservingBlock
from micado.templates.MicadoImgCalDark import MicadoImgCalDark
from micado.test.functional.detector_test import generate_detectorarray
from micado.util.Processor import LocalProcessor


def test_dark_recipe():
    """Test the creation of Dark through the lpu/dpu recipe."""
    # Hardware and observation setup.
    now = datetime.now()
    dark_real = 3.0
    size = 30

    # Template start doubles as RNG seed.
    dt_tpl__start = now

    # Detector array
    mydetectorarray = generate_detectorarray(dark=dark_real, size=size, n_det=1)

    # A template
    dark_template = MicadoImgCalDark(det__dit=1, det__ndit=1)

    # An observingblock
    dark_ob = ObservingBlock(
        name="MyDarkTest",
        instrument="MICADO",
        target="empty_sky",
        templates=[dark_template],
    )

    mydetectorarray.commit()
    dark_template.commit()
    dark_ob.commit()

    # Generate a dark through the recipe.
    lpu = LocalProcessor()
    lpu.run(
        "rawdark",
        detector=mydetectorarray,
        template=dark_template,
        observing_block=dark_ob,
        tpl__start=dt_tpl__start.isoformat(),
        commit=True,
        skip_store=True,
    )
    # In case of dpu do:
    # jobid = dpu.run(....)
    # dpu.wait_for_jobs()
    # dark_through_lpu = dpu.get_job_result(jobid)[0][0]

    dark_through_lpu = lpu.get_job_result()[0]

    # Generate a dark here.
    # DarkExtensions, this is probably something that should be moved to the make method though
    myextensions = [
        DarkExtension(detector=det, BITPIX=-64, NAXIS=2)
        for det in mydetectorarray.detector_array_list
    ]

    dark_manually = Dark(
        detector=mydetectorarray,
        template=dark_template,
        observing_block=dark_ob,
        tpl__start=dt_tpl__start.timestamp(),
        mjdobs=dt_tpl__start.timestamp(),
        extensions=myextensions,
    )
    dark_manually.make()
    # Saving the FITS file might overwrite the file created on the DPU. However,
    # this should not matter because the filename contains a hash of the
    # contents.
    dark_manually.save_fits()

    # Tests
    assert dark_through_lpu.observing_block is dark_ob
    assert dark_through_lpu.template is dark_template
    assert dark_through_lpu.detector is mydetectorarray

    # The tpl__start should ensure all the pixel values are identical.
    assert dark_through_lpu == dark_manually, dark_through_lpu.get_differences(
        dark_manually
    )

    # Cleanup.
    # Delete test objects from database.
    to_delete = (
        [
            dark_manually,
            dark_through_lpu,
            mydetectorarray,
            dark_template,
            dark_ob,
        ]
        + dark_manually.extensions
        + dark_through_lpu.extensions
        + mydetectorarray.detector_array_list
    )
    for obj in to_delete:
        if obj.persists():
            # TODO: Only delete objects that were actually created during this test.
            context.delete(obj, True)
