"""Check whether all tex files are actually included."""

from pathlib import Path

dir_tex = Path(__file__).parent.parent.parent.parent / "docs"

# Find all tex files.
paths_tex = list(dir_tex.glob("**/*.tex"))
stems_tex = [p.stem for p in paths_tex]

# Check for duplicate file names.
assert len(stems_tex) == len(set(stems_tex))

# Find all include commands.
# Note that circular includes are not detected this way.
lines_inc = [
    line.strip()
    for p in paths_tex
    for line in open(p).readlines()
    if ("\\include{" in line or "\\input{" in line) and line.strip()[0] != "%"
]

# Get the included filenames.
# '\\input{InstrumentDataRates}' -> 'InstrumentDataRates'
stems_inc = [
    line.split("{")[1].split("}")[0].split("/")[-1].split(".tex")[0]
    for line in lines_inc
]

# Assert that all included files exist.
stems_missing = set(stems_inc) - set(stems_tex)
assert not stems_missing, stems_missing

stems_included_multiple_times = [
    stem for stem in stems_inc if stems_inc.count(stem) > 1
]

assert (
    not stems_included_multiple_times
), f"Error, tex files included multiple times: {stems_included_multiple_times}"

stems_notincluded = set(stems_tex) - set(stems_inc)

stems_expected_missing = {
    # Name of the document
    "MicadoWISE_DataModel",
}

stems_missing = stems_notincluded - stems_expected_missing
assert not stems_missing, f"Warning, tex files not imported: {stems_missing}"
