# -*- coding: utf-8 -*-
"""Create class inheritance diagram.

First do both
conda install graphviz
pip install graphviz
"""

import copy

from graphviz import Digraph

from micado.main.commonwrapper import classcache
from micado.main.DataItem import DataItem
from micado.main.Ext import Ext
from micado.main.Final import Final
from micado.main.RecipeStepExtra import RecipeStepBase
from micado.main.Static import Static
from micado.main.StaticTable import StaticTable

# aweimports fills the classcache.
# noinspection PyUnresolvedReferences
import micado.main.aweimports  # noqa NOSONAR

dot = Digraph(comment="MICADO Classes")
dot.attr(rankdir="RL")
dot.attr("node", shape="box")
dot.format = "svg"
classes_all = [c.aclass for c in classcache.values()]
classes_dataitems = [
    c
    for c in classes_all
    if issubclass(c, DataItem)
    and not issubclass(c, (RecipeStepBase, Final, Static, Ext, StaticTable))
    and not c.__name__.endswith("Extra")
]

classes = copy.copy(classes_dataitems)
while classes:
    cls = classes.pop()
    assert len(cls.__bases__) == 1
    sup = cls.base_doc()
    if sup is not None:
        dot.edge(cls.name_eso(includebase=True), sup.name_eso(includebase=True))

dot.render("docs/source/generated/imaging_inheritance_diagram", view=False)
