"""Tools for interfacing with the IRDB."""
import os
from pathlib import Path

# import irdb
import scopesim.rc
import yaml

# PATH_HERE = Path(__file__).resolve().parent
# Path in the repository where the IRDB git-submodule is installed.
# PATH_IRDB_DEV = PATH_HERE.parent.parent.parent / "irdb"
# Path in site-packages where the IRDB is copied to.
# PATH_IRDB_PROD = PATH_HERE.parent.parent / "irdb"

# if os.path.exists(PATH_IRDB_DEV):
#    PATH_IRDB = PATH_IRDB_DEV
# elif os.path.exists(PATH_IRDB_PROD):
#    PATH_IRDB = PATH_IRDB_PROD
# else:
#    raise ValueError(f"Cannot find IRDB: {PATH_IRDB_DEV} or {PATH_IRDB_PROD}")

# PATH_IRDB = Path(irdb.__path__[0])


def get_path_irdb(needed=["Armazones", "ELT", "MICADO", "LFOA", "MAORY"]):
    """return path to irdb"""
    for test_path in scopesim.rc.__search_path__:
        if os.path.exists(test_path):
            entries = os.listdir(test_path)
            # are X needed entries present in the test_path?
            if len(set(entries)) - len(set(entries) - set(needed)) == len(needed):
                return Path(test_path)
    raise NotImplementedError(f"irdb path not found for packages: {str(needed)}")


PATH_IRDB = get_path_irdb()

# Fill search path so relevant .dat files can be found in python_from_vodml.py
for name in ["MICADO", "Armazones", "ELT", "MICADO"]:
    scopesim.rc.__search_path__.append(str(PATH_IRDB / name))
    scopesim.rc.__search_path__.append(str(PATH_IRDB / name / "filters"))

# TODO: Get all yaml files from from MICADO_default.yaml.
INSTRUMENT = yaml.safe_load(open(PATH_IRDB / "MICADO" / "MICADO.yaml"))
ATMOSPHERE = yaml.safe_load(open(PATH_IRDB / "Armazones" / "Armazones.yaml"))
TELESCOPE = yaml.safe_load(open(PATH_IRDB / "ELT" / "ELT.yaml"))
RELAYOPTICS = yaml.safe_load(open(PATH_IRDB / "MICADO" / "MICADO_Standalone_RO.yaml"))
DETECTORARRAY = yaml.safe_load(open(PATH_IRDB / "MICADO" / "MICADO_H4RG.yaml"))

YAMLS_IRDB = [INSTRUMENT, ATMOSPHERE, TELESCOPE, RELAYOPTICS, DETECTORARRAY]

IRDB_FROM_MW = {
    "DetectorArray": {
        "detector_array_list": "full_detector_array",
        "linearity": "detector_linearity",
        "reference_pixel_border": "border_reference_pixels",
    },
    "Telescope": {
        "reflection": "telescope_reflection",
    },
    "Instrument": {
        "static_surfaces": "micado_static_surfaces",
        "ncpas_psf": "micado_ncpas_psf",
    },
    "RelayOptics": {
        "psf": "relay_psf",
        "surface_list": "relay_surface_list",
    },
    "Atmosphere": {},
}

MW_FROM_IRDB = {khw: {v: k for k, v in hw.items()} for khw, hw in IRDB_FROM_MW.items()}


def get_effect(irdb_object: dict, name_effect: str):
    """Get effect from IRDB yaml."""
    # TODO: make this nicer
    # name_effect_irdb = IRDB_FROM_MW.get(name_effect, name_effect)
    irdb_from_mw_flat = {k: v for vv in IRDB_FROM_MW.values() for k, v in vv.items()}
    name_effect_irdb = irdb_from_mw_flat.get(name_effect, name_effect)
    effects = [
        effect
        for effect in irdb_object["effects"]
        if effect["name"] == name_effect_irdb
    ] + [None]
    return effects[0]


def irdb_from_class_attr(classname: str, attrname: str):
    """Get the irdb yaml for a given classname and attrname."""
    name_effect_irdb = IRDB_FROM_MW.get(classname, {}).get(attrname, attrname)
    effects = [
        effect
        for irdb_object in YAMLS_IRDB
        for effect in irdb_object["effects"]
        if effect["name"] == name_effect_irdb
    ] + [None]
    assert len(effects) < 3
    return effects[0]
